@@ -269,10 +269,12 @@ SV *svn_swig_pl_array_to_list(const apr_array_header_t *array)
     return convert_array (array, (element_converter_t)convert_string, NULL);
 }
 
+/* Formerly used by pre-1.0 APIs. Now unused
 SV *svn_swig_pl_ints_to_list(const apr_array_header_t *array)
 {
     return convert_array (array, (element_converter_t)convert_int, NULL);
 }
+*/
 
 SV *svn_swig_pl_convert_array(const apr_array_header_t *array,
                               swig_type_info *tinfo)
@@ -438,8 +440,6 @@ static item_baton * make_baton(apr_pool_t *pool,
 {
     item_baton *newb = apr_palloc(pool, sizeof(*newb));
 
-    SvREFCNT_inc(editor);
-
     newb->editor = editor;
     newb->baton = baton;
 
@@ -463,7 +463,6 @@ static svn_error_t * close_baton(void *baton, const char *method, apr_pool_t *po
                                              "OS", ib->editor, pool, POOLINFO));
     }
 
-    SvREFCNT_dec(ib->editor);
     return SVN_NO_ERROR;
 }
 
@@ -714,7 +713,6 @@ static svn_error_t * thunk_close_file(void *file_baton,
 				         ib->editor, ib->baton, text_checksum,
 				         pool, POOLINFO));
 
-    SvREFCNT_dec(ib->editor);
     SvREFCNT_dec(ib->baton);
 
     return SVN_NO_ERROR;
@@ -772,6 +770,7 @@ void svn_delta_make_editor(svn_delta_editor_t **editor,
 
     *editor = thunk_editor;
     *edit_baton = make_baton(pool, perl_editor, NULL);
+    svn_swig_pl_hold_ref_in_pool (pool, perl_editor);
 }
 
 svn_error_t *svn_swig_pl_thunk_log_receiver(void *baton,
@@ -1212,17 +1211,9 @@ svn_boolean_t svn_swig_pl_thunk_config_enumerator (const char *name, const char
 
 
 /* default pool support */
-static apr_pool_t *current_pool = 0;
 
-apr_pool_t *svn_swig_pl_get_current_pool (void)
-{
-  return current_pool;
-}
-
-void svn_swig_pl_set_current_pool (apr_pool_t *pool)
-{
-  current_pool = pool;
-}
+apr_pool_t *svn_swig_pl_get_current_pool (void);
+void svn_swig_pl_set_current_pool (apr_pool_t *pool);
 
 apr_pool_t *svn_swig_pl_make_pool (SV *obj)
 {
@@ -1238,12 +1229,11 @@ apr_pool_t *svn_swig_pl_make_pool (SV *obj)
 	}
     }
 
-    if (!current_pool)
+    if (!svn_swig_pl_get_current_pool())
 	svn_swig_pl_callback_thunk (CALL_METHOD, (void *)"new_default",
 			            &obj, "s", "SVN::Pool");
-    pool = current_pool;
 
-    return pool;
+    return svn_swig_pl_get_current_pool();
 }
 
 /* stream interpolability with io::handle */
@@ -1344,7 +1334,7 @@ svn_error_t *svn_swig_pl_make_stream (svn_stream_t **stream, SV *obj)
 
     if (obj && SvROK(obj) && SvTYPE(SvRV(obj)) == SVt_PVGV &&
 	(io = GvIO(SvRV(obj)))) {
-	apr_pool_t *pool = current_pool;
+	apr_pool_t *pool = svn_swig_pl_get_current_pool();
 	io_baton_t *iob = apr_palloc (pool, sizeof(io_baton_t));
 	SvREFCNT_inc (obj);
 	iob->obj = obj;
