@@ -16,7 +16,7 @@
  * @endcopyright
  *
  * @file svn_io.h
- * @brief general Subversion I/O definitions
+ * @brief General file I/O for Subversion
  */
 
 /* ==================================================================== */
@@ -59,6 +59,17 @@ svn_error_t *svn_io_check_path (const char *path,
                                 svn_node_kind_t *kind,
                                 apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Like svn_io_check_path(), but also set *is_special to @c TRUE if
+ * the path is not a normal file.
+ */
+svn_error_t *svn_io_check_special_path (const char *path,
+                                        svn_node_kind_t *kind,
+                                        svn_boolean_t *is_special,
+                                        apr_pool_t *pool);
+
 /** Like svn_io_check_path(), but resolve symlinks.  This returns the
     same varieties of @a kind as svn_io_check_path(). */ 
 svn_error_t *svn_io_check_resolved_path (const char *path,
@@ -73,8 +84,6 @@ svn_error_t *svn_io_check_resolved_path (const char *path,
  * then the @c APR_DELONCLOSE flag will be used when opening the file. The
  * @c APR_BUFFERED flag will always be used.
  *
- * @a suffix may not be null.
- *
  * The first attempt will just append @a suffix.  If the result is not
  * a unique name, then subsequent attempts will append a dot,
  * followed by an iteration number ("2", then "3", and so on),
@@ -119,8 +128,31 @@ svn_error_t *svn_io_open_unique_file (apr_file_t **f,
                                       svn_boolean_t delete_on_close,
                                       apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Like svn_io_open_unique_file, except that instead of creating a
+ * file, a symlink is generated that references the path @a dest.
+ */
+svn_error_t *svn_io_create_unique_link (const char **unique_name_p,
+                                        const char *path,
+                                        const char *dest,
+                                        const char *suffix,
+                                        apr_pool_t *pool);
 
-/** Set @a dir to a directory path (allocated in @a pool) deemed
+
+/**
+ * @since New in 1.1.
+ *
+ * Set @a *dest to the path that the symlink at @a path references.
+ * Allocate the string from @a pool.
+ */
+svn_error_t *svn_io_read_link (svn_string_t **dest,
+                               const char *path,
+                               apr_pool_t *pool);
+
+
+/** Set @a *dir to a directory path (allocated in @a pool) deemed
  * usable for the creation of temporary files and subdirectories.
  */
 svn_error_t *svn_io_temp_dir (const char **dir,
@@ -136,6 +168,18 @@ svn_error_t *svn_io_copy_file (const char *src,
                                svn_boolean_t copy_perms,
                                apr_pool_t *pool);
 
+/** 
+ * @since New in 1.1.
+ *
+ * Copy symbolic link @a src to @a dst atomically.  Overwrite @a dst
+ * if it exists, else create it.  Both @a src and @a dst are
+ * utf8-encoded filenames.  After copying, the @a dst link will point
+ * to the same thing @a src does.
+ */
+svn_error_t *svn_io_copy_link (const char *src,
+                               const char *dst,
+                               apr_pool_t *pool);
+
 
 /** Recursively copy directory @a src into @a dst_parent, as a new entry named
  * @a dst_basename.  If @a dst_basename already exists in @a dst_parent, 
@@ -197,7 +241,7 @@ svn_error_t *svn_io_set_file_read_only (const char *path,
 /** Make a file as writable as the operating system allows.
  * @a path is the utf8-encoded path to the file.  If @a ignore_enoent is
  * @c TRUE, don't fail if the target file doesn't exist.
- * @warning On Unix this function will do the equivlanet of chmod a+w path.
+ * @warning On Unix this function will do the equivalent of chmod a+w path.
  * If this is not what you want you should not use this function, but rather
  * use apr_file_perms_set().
  */
@@ -205,11 +249,38 @@ svn_error_t *svn_io_set_file_read_write (const char *path,
                                          svn_boolean_t ignore_enoent,
                                          apr_pool_t *pool);
 
+
+/** Minimally change the read-write permissions of a file.
+ * @since New in 1.1.
+ *
+ * When making @a path read-write on operating systems with unix style
+ * permissions, set the permissions on @a path to the permissions that
+ * are set when a new file is created (effectively honoring the user's
+ * umask).
+ *
+ * When making the file read-only on operating systems with unix style
+ * permissions, remove all write permissions.
+ *
+ * On other operating systems, toggle the file's "writability" as much as
+ * the operating system allows.
+ *
+ * @a path is the utf8-encoded path to the file.  If @a enable_write
+ * is @c TRUE, then make the file read-write.  If @c FALSE, make it
+ * write-only.  If @a ignore_enoent is @c TRUE, don't fail if the target
+ * file doesn't exist.
+ */
+svn_error_t *svn_io_set_file_read_write_carefully (const char *path,
+                                                   svn_boolean_t enable_write,
+                                                   svn_boolean_t ignore_enoent,
+                                                   apr_pool_t *pool);
+    
 /** Toggle a file's "executability".
  *
  * When making the file executable on operating systems with unix style
- * permissions only make the file executable for the user, group or world
- * if the user, group or world already have permission to read the file.
+ * permissions, never add an execute permission where there is not
+ * already a read permission: that is, only make the file executable
+ * for the user, group or world if the corresponding read permission
+ * is already set for user, group or world.
  *
  * When making the file non-executable on operating systems with unix style
  * permissions, remove all execute permissions.
@@ -297,7 +368,7 @@ svn_error_t *svn_io_file_checksum (unsigned char digest[],
                                    apr_pool_t *pool);
 
 
-/* Set @a *same to non-zero if @a file1 and @a file2 have the same
+/** Set @a *same to non-zero if @a file1 and @a file2 have the same
  * contents, else set it to zero.  Use @a pool for temporary allocations.
  */
 svn_error_t *svn_io_files_contents_same_p (svn_boolean_t *same,
@@ -313,15 +384,43 @@ svn_error_t *svn_io_file_create (const char *file,
                                  const char *contents,
                                  apr_pool_t *pool);
 
-/** Lock file at @a lock_file. If @exclusive is TRUE,
+/**
+ * @deprecated Provided for backward compatibility with the 1.0 API.
+ *
+ * Lock file at @a lock_file. If @a exclusive is TRUE,
  * obtain exclusive lock, otherwise obtain shared lock.
- * Lock will be automaticaly released when @a pool is cleared or destroyed.
+ * Lock will be automatically released when @a pool is cleared or destroyed.
  * Use @a pool for memory allocations.
  */
 svn_error_t *svn_io_file_lock (const char *lock_file,
                                svn_boolean_t exclusive,
                                apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Lock file at @a lock_file. If @a exclusive is TRUE,
+ * obtain exclusive lock, otherwise obtain shared lock.
+ *
+ * If @a nonblocking is TRUE, do not wait for the lock if it
+ * is not available: throw an error instead.
+ *
+ * Lock will be automatically released when @a pool is cleared or destroyed.
+ * Use @a pool for memory allocations.
+ */
+svn_error_t *svn_io_file_lock2 (const char *lock_file,
+                                svn_boolean_t exclusive,
+                                svn_boolean_t nonblocking,
+                                apr_pool_t *pool);
+/**
+ * @since New in 1.1.
+ *
+ * Flush any unwritten data from @a file to disk.  Use @a pool for
+ *  memory allocations.
+ */
+svn_error_t *svn_io_file_flush_to_disk (apr_file_t *file,
+                                        apr_pool_t *pool);
+
 /** Copy file @a file from location @a src_path to location @a dest_path.
  * Use @a pool for memory allocations.
  */
@@ -465,6 +564,15 @@ svn_stream_readline (svn_stream_t *stream,
                      svn_boolean_t *eof,
                      apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Read the contents of the readable stream @a from and write them to the
+ * writable stream @a to.
+ */
+svn_error_t *svn_stream_copy (svn_stream_t *from, svn_stream_t *to,
+                              apr_pool_t *pool);
+
 /** @} */
 
 /** Sets @a *result to a string containing the contents of @a filename, a
@@ -501,7 +609,7 @@ svn_error_t *svn_io_remove_dir (const char *path, apr_pool_t *pool);
 
 
 /** Read all of the disk entries in directory @a path, a utf8-encoded
- * path.  Return a @a dirents hash mapping dirent names (<tt>char *</tt>) to
+ * path.  Set @a *dirents to a hash mapping dirent names (<tt>char *</tt>) to
  * enumerated dirent filetypes (@c svn_node_kind_t *).
  *
  * Note:  the `.' and `..' directories normally returned by
@@ -547,9 +655,9 @@ svn_error_t *svn_io_dir_walk (const char *dirname,
  * @a exitcode is not set and the exit code is non-zero, then an 
  * @c SVN_ERR_EXTERNAL_PROGRAM error will be returned.
  *
- * @a args is a list of utf8-encoded (<tt>const char *</tt>)'s, terminated by
- * @c NULL.  @c ARGS[0] is the name of the program, though it need not be
- * the same as @a cmd.
+ * @a args is a list of utf8-encoded <tt>const char *</tt> arguments,
+ * terminated by @c NULL.  @a args[0] is the name of the program, though it
+ * need not be the same as @a cmd.
  *
  * @a inherit sets whether the invoked program shall inherit its environment or
  * run "clean".
@@ -640,7 +748,7 @@ svn_error_t *svn_io_run_diff3 (const char *dir,
 
 /** Examine utf8-encoded @a file to determine if it can be described by a
  * known (as in, known by this function) Multipurpose Internet Mail
- * Extension (MIME) type.  If so, set @a mimetype to a character string
+ * Extension (MIME) type.  If so, set @a *mimetype to a character string
  * describing the MIME type, else set it to @c NULL.  Use @a pool for any
  * necessary allocations.
  */
@@ -678,7 +786,7 @@ svn_io_file_read (apr_file_t *file, void *buf,
                   apr_size_t *nbytes, apr_pool_t *pool);
 
 
-/** Wrapper for @c apr_file_write_full(), which see. */
+/** Wrapper for @c apr_file_read_full(), which see. */
 svn_error_t *
 svn_io_file_read_full (apr_file_t *file, void *buf, 
                        apr_size_t nbytes, apr_size_t *bytes_read,
@@ -728,6 +836,19 @@ svn_error_t *
 svn_io_dir_make_hidden (const char *path, apr_fileperms_t perm,
                         apr_pool_t *pool);
 
+/**
+ * @since New in 1.1.
+ *
+ * Same as svn_io_dir_make, but attempts to set the sgid on the
+ * directory on systems that support it.  Does not return an error if
+ * the attempt to set the sgid bit fails.  On Unix filesystems,
+ * setting the sgid bit on a directory ensures that files and
+ * subdirectories created within inherit group ownership from the
+ * parent instead of from the primary gid. */
+svn_error_t *
+svn_io_dir_make_sgid (const char *path, apr_fileperms_t perm,
+                      apr_pool_t *pool);
+
 /** Wrapper for @c apr_dir_open(), which see.  @a dirname is utf8-encoded. */
 svn_error_t *
 svn_io_dir_open (apr_dir_t **new_dir, const char *dirname, apr_pool_t *pool);
@@ -770,7 +891,8 @@ svn_io_read_version_file (int *version, const char *path, apr_pool_t *pool);
 
 /** Create (or overwrite) the file at @a path with new contents,
  * formatted as a non-negative integer @a version followed by a single
- * newline.  Use @a pool for all allocations.
+ * newline.  On successful completion the file will be read-only.  Use
+ * @a pool for all allocations.
  */
 svn_error_t *
 svn_io_write_version_file (const char *path, int version, apr_pool_t *pool);
