@@ -31,7 +31,6 @@
 
 #include "svn_types.h"
 #include "svn_string.h"
-#include "svn_error.h"
 #include "svn_delta.h"
 #include "svn_ra.h"
 #include "svn_dav.h"
@@ -117,24 +116,60 @@ typedef int svn_ra_dav__xml_endelm_cb(void *userdata,
 
 
 
+
+
+/* Context for neon request hooks; shared by the neon callbacks in
+   session.c.  */
+struct lock_request_baton
+{
+  /* The method neon is about to execute. */
+  const char *method;
+
+  /* The current working revision of item being locked. */
+  svn_revnum_t current_rev;
+
+  /* Whether client is "forcing" a lock or unlock. */
+  svn_boolean_t force;
+
+  /* The creation-date returned for newly created lock. */
+  apr_time_t creation_date;
+
+  /* The person who created the lock. */
+  const char *lock_owner;
+
+  /* A parser for handling <D:error> responses from mod_dav_svn. */
+  ne_xml_parser *error_parser;
+
+  /* If <D:error> is returned, here's where the parsed result goes. */
+  svn_error_t *err;
+
+  /* A place for allocating fields in this structure. */
+  apr_pool_t *pool;
+};
+
+
 typedef struct {
   apr_pool_t *pool;
-
   const char *url;                      /* original, unparsed session url */
   ne_uri root;                          /* parsed version of above */
   const char *repos_root;               /* URL for repository root */
 
   ne_session *sess;                     /* HTTP session to server */
   ne_session *sess2;
-  
+
   const svn_ra_callbacks_t *callbacks;  /* callbacks to get auth data */
   void *callback_baton;
-
+ 
   svn_auth_iterstate_t *auth_iterstate; /* state of authentication retries */
+  const char *auth_username;            /* last authenticated username used */
 
   svn_boolean_t compression;            /* should we use http compression? */
   const char *uuid;                     /* repository UUID */
-} svn_ra_session_t;
+
+  
+  struct lock_request_baton *lrb;       /* used by lock/unlock */
+
+} svn_ra_dav__session_t;
 
 
 /* Id used with ne_set_session_private() and ne_get_session_private()
@@ -149,45 +184,47 @@ typedef struct {
 #endif
 
 
-/** plugin function prototypes */
+/** vtable function prototypes */
 
-svn_error_t *svn_ra_dav__get_latest_revnum(void *session_baton,
+svn_error_t *svn_ra_dav__get_latest_revnum(svn_ra_session_t *session,
                                            svn_revnum_t *latest_revnum,
                                            apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__get_dated_revision (void *session_baton,
+svn_error_t *svn_ra_dav__get_dated_revision (svn_ra_session_t *session,
                                              svn_revnum_t *revision,
                                              apr_time_t timestamp,
                                              apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__change_rev_prop (void *session_baton,
+svn_error_t *svn_ra_dav__change_rev_prop (svn_ra_session_t *session,
                                           svn_revnum_t rev,
                                           const char *name,
                                           const svn_string_t *value,
                                           apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__rev_proplist (void *session_baton,
+svn_error_t *svn_ra_dav__rev_proplist (svn_ra_session_t *session,
                                        svn_revnum_t rev,
                                        apr_hash_t **props,
                                        apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__rev_prop (void *session_baton,
+svn_error_t *svn_ra_dav__rev_prop (svn_ra_session_t *session,
                                    svn_revnum_t rev,
                                    const char *name,
                                    svn_string_t **value,
                                    apr_pool_t *pool);
 
 svn_error_t * svn_ra_dav__get_commit_editor(
-  void *session_baton,
+  svn_ra_session_t *session,
   const svn_delta_editor_t **editor,
   void **edit_baton,
   const char *log_msg,
   svn_commit_callback_t callback,
   void *callback_baton,
+  apr_hash_t *lock_tokens,
+  svn_boolean_t keep_locks,
   apr_pool_t *pool);
 
 svn_error_t * svn_ra_dav__get_file(
-  void *session_baton,
+  svn_ra_session_t *session,
   const char *path,
   svn_revnum_t revision,
   svn_stream_t *stream,
@@ -196,7 +233,7 @@ svn_error_t * svn_ra_dav__get_file(
   apr_pool_t *pool);
 
 svn_error_t *svn_ra_dav__get_dir(
-  void *session_baton,
+  svn_ra_session_t *session,
   const char *path,
   svn_revnum_t revision,
   apr_hash_t **dirents,
@@ -209,8 +246,8 @@ svn_error_t * svn_ra_dav__abort_commit(
  void *edit_baton);
 
 svn_error_t * svn_ra_dav__do_update(
-  void *session_baton,
-  const svn_ra_reporter_t **reporter,
+  svn_ra_session_t *session,
+  const svn_ra_reporter2_t **reporter,
   void **report_baton,
   svn_revnum_t revision_to_update_to,
   const char *update_target,
@@ -220,8 +257,8 @@ svn_error_t * svn_ra_dav__do_update(
   apr_pool_t *pool);
 
 svn_error_t * svn_ra_dav__do_status(
-  void *session_baton,
-  const svn_ra_reporter_t **reporter,
+  svn_ra_session_t *session,
+  const svn_ra_reporter2_t **reporter,
   void **report_baton,
   const char *status_target,
   svn_revnum_t revision,
@@ -231,8 +268,8 @@ svn_error_t * svn_ra_dav__do_status(
   apr_pool_t *pool);
 
 svn_error_t * svn_ra_dav__do_switch(
-  void *session_baton,
-  const svn_ra_reporter_t **reporter,
+  svn_ra_session_t *session,
+  const svn_ra_reporter2_t **reporter,
   void **report_baton,
   svn_revnum_t revision_to_update_to,
   const char *update_target,
@@ -243,8 +280,8 @@ svn_error_t * svn_ra_dav__do_switch(
   apr_pool_t *pool);
 
 svn_error_t * svn_ra_dav__do_diff(
-  void *session_baton,
-  const svn_ra_reporter_t **reporter,
+  svn_ra_session_t *session,
+  const svn_ra_reporter2_t **reporter,
   void **report_baton,
   svn_revnum_t revision,
   const char *diff_target,
@@ -256,10 +293,11 @@ svn_error_t * svn_ra_dav__do_diff(
   apr_pool_t *pool);
 
 svn_error_t * svn_ra_dav__get_log(
-  void *session_baton,
+  svn_ra_session_t *session,
   const apr_array_header_t *paths,
   svn_revnum_t start,
   svn_revnum_t end,
+  int limit,
   svn_boolean_t discover_changed_paths,
   svn_boolean_t strict_node_history,
   svn_log_message_receiver_t receiver,
@@ -267,12 +305,28 @@ svn_error_t * svn_ra_dav__get_log(
   apr_pool_t *pool);
 
 svn_error_t *svn_ra_dav__do_check_path(
-  void *session_baton,
+  svn_ra_session_t *session,
   const char *path,
   svn_revnum_t revision,
   svn_node_kind_t *kind,
   apr_pool_t *pool);
 
+svn_error_t *svn_ra_dav__do_stat(
+  svn_ra_session_t *session,
+  const char *path,
+  svn_revnum_t revision,
+  svn_dirent_t **dirent,
+  apr_pool_t *pool);
+
+svn_error_t *svn_ra_dav__get_file_revs (svn_ra_session_t *session,
+                                        const char *path,
+                                        svn_revnum_t start,
+                                        svn_revnum_t end,
+                                        svn_ra_file_rev_handler_t handler,
+                                        void *handler_baton,
+                                        apr_pool_t *pool);
+
+
 /*
 ** SVN_RA_DAV__LP_*: local properties for RA/DAV
 **
@@ -448,11 +502,13 @@ svn_error_t *svn_ra_dav__get_vcc(const char **vcc,
 /* Issue a PROPPATCH request on URL, transmitting PROP_CHANGES (a hash
    of const svn_string_t * values keyed on Subversion user-visible
    property names) and PROP_DELETES (an array of property names to
-   delete).  Use POOL for all allocations.  */
-svn_error_t *svn_ra_dav__do_proppatch (svn_ra_session_t *ras,
+   delete).  Send any extra request headers in EXTRA_HEADERS. Use POOL
+   for all allocations.  */
+svn_error_t *svn_ra_dav__do_proppatch (svn_ra_dav__session_t *ras,
                                        const char *url,
                                        apr_hash_t *prop_changes,
                                        apr_array_header_t *prop_deletes,
+                                       apr_hash_t *extra_headers,
                                        apr_pool_t *pool);
 
 extern const ne_propname svn_ra_dav__vcc_prop;
@@ -464,7 +520,7 @@ extern const ne_propname svn_ra_dav__checked_in_prop;
 /* send an OPTIONS request to fetch the activity-collection-set */
 svn_error_t * svn_ra_dav__get_activity_collection(
   const svn_string_t **activity_coll,
-  svn_ra_session_t *ras,
+  svn_ra_dav__session_t *ras,
   const char *url,
   apr_pool_t *pool);
 
@@ -507,6 +563,9 @@ svn_ra_dav__lookup_xml_elem(const svn_ra_dav__xml_elm_t *table,
  * STATUS_CODE is an optional 'out' parameter; if non-NULL, then set
  * *STATUS_CODE to the http status code returned by the server.
  *
+ * If SPOOL_RESPONSE is set, the request response will be cached to
+ * disk in a tmpfile (in full), then read back and parsed.
+ *
  * Use POOL for any temporary allocation.
  */
 svn_error_t *
@@ -523,6 +582,7 @@ svn_ra_dav__parsed_request(ne_session *sess,
                            void *baton,
                            apr_hash_t *extra_headers,
                            int *status_code,
+                           svn_boolean_t spool_response,
                            apr_pool_t *pool);
   
 
@@ -548,6 +608,7 @@ svn_ra_dav__parsed_request_compat(ne_session *sess,
                                   void *baton,
                                   apr_hash_t *extra_headers,
                                   int *status_code,
+                                  svn_boolean_t spool_response,
                                   apr_pool_t *pool);
 
 
@@ -626,7 +687,20 @@ enum {
   ELEM_name_creator_displayname,
   ELEM_svn_error,
   ELEM_human_readable,
-  ELEM_repository_uuid
+  ELEM_repository_uuid,
+  ELEM_get_locations_report,
+  ELEM_location,
+  ELEM_file_revs_report,
+  ELEM_file_rev,
+  ELEM_rev_prop,
+  ELEM_get_locks_report,
+  ELEM_lock,
+  ELEM_lock_path,
+  ELEM_lock_token,
+  ELEM_lock_owner,
+  ELEM_lock_comment,
+  ELEM_lock_creationdate,
+  ELEM_lock_expirationdate
 };
 
 /* ### docco */
@@ -634,10 +708,12 @@ svn_error_t * svn_ra_dav__merge_activity(
     svn_revnum_t *new_rev,
     const char **committed_date,
     const char **committed_author,
-    svn_ra_session_t *ras,
+    svn_ra_dav__session_t *ras,
     const char *repos_url,
     const char *activity_url,
     apr_hash_t *valid_targets,
+    apr_hash_t *lock_tokens,
+    svn_boolean_t keep_locks,
     svn_boolean_t disable_merge_response,
     apr_pool_t *pool);
 
@@ -655,15 +731,27 @@ void svn_ra_dav__copy_href(svn_stringbuf_t *dst, const char *src);
 /* If RAS contains authentication info, attempt to store it via client
    callbacks.  */
 svn_error_t *
-svn_ra_dav__maybe_store_auth_info (svn_ra_session_t *ras);
+svn_ra_dav__maybe_store_auth_info (svn_ra_dav__session_t *ras);
+
+
+/* Like svn_ra_dav__maybe_store_auth_info(), but conditional on ERR.
+
+   Attempt to store auth info only if ERR is NULL or if ERR->apr_err
+   is not SVN_ERR_RA_NOT_AUTHORIZED.  If ERR is not null, return it no
+   matter what, otherwise return the result of the attempt (if any) to
+   store auth info, else return SVN_NO_ERROR. */
+svn_error_t *
+svn_ra_dav__maybe_store_auth_info_after_result(svn_error_t *err,
+                                               svn_ra_dav__session_t *ras);
 
 
 /* Create an error object for an error from neon in the given session,
    where the return code from neon was RETCODE, and CONTEXT describes
-   what was being attempted. */
+   what was being attempted.  Do temporary allocations in POOL. */
 svn_error_t *svn_ra_dav__convert_error(ne_session *sess,
                                        const char *context,
-                                       int retcode);
+                                       int retcode,
+                                       apr_pool_t *pool);
 
 
 /* Given a neon REQUEST and SESSION, run the request; if CODE_P is
@@ -697,6 +785,68 @@ svn_ra_dav__request_dispatch(int *code_p,
                              apr_pool_t *pool);
 
 
+/* Give PARSER the ability to parse a mod_dav_svn <D:error> response
+   body in the case of a non-2XX response to REQUEST.  If a <D:error>
+   response is detected, then set *ERR to the parsed error.
+*/
+void
+svn_ra_dav__add_error_handler(ne_request *request,
+                              ne_xml_parser *parser,
+                              svn_error_t **err,
+                              apr_pool_t *pool);
+
+
+/*
+ * Implements the get_locations RA layer function. */
+svn_error_t *
+svn_ra_dav__get_locations (svn_ra_session_t *session,
+                           apr_hash_t **locations,
+                           const char *path,
+                           svn_revnum_t peg_revision,
+                           apr_array_header_t *location_revisions,
+                           apr_pool_t *pool);
+
+
+/*
+ * Implements the get_locks RA layer function. */
+svn_error_t *
+svn_ra_dav__get_locks(svn_ra_session_t *session,
+                      apr_hash_t **locks,
+                      const char *path,
+                      apr_pool_t *pool);
+
+/*
+ * Implements the get_lock RA layer function. */
+svn_error_t *
+svn_ra_dav__get_lock(svn_ra_session_t *session,
+                     svn_lock_t **lock,
+                     const char *path,
+                     apr_pool_t *pool);
+
+
+/* Helper function.  Loop over LOCK_TOKENS and assemble all keys and
+   values into a stringbuf allocated in POOL.  The string will be of
+   the form
+
+    <S:lock-token-list xmlns:S="svn:">
+      <S:lock>
+        <S:lock-path>path</S:lock-path>
+        <S:lock-token>token</S:lock-token>
+      </S:lock>
+      [...]
+    </S:lock-token-list>
+
+   Callers can then send this in the request bodies, as a way of
+   reliably marshalling potentially unbounded lists of locks.  (We do
+   this because httpd has limits on how much data can be sent in 'If:'
+   headers.)
+ */
+svn_error_t *
+svn_ra_dav__assemble_locktoken_body(svn_stringbuf_t **body,
+                                    apr_hash_t *lock_tokens,
+                                    apr_pool_t *pool);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
