@@ -28,6 +28,9 @@
 #include "svn_path.h"
 #include "svn_repos.h"
 #include "svn_pools.h"
+#include "svn_props.h"
+#include "svn_private_config.h"
+#include "repos.h"
 
 
 
@@ -174,7 +177,7 @@ authz_root_check (svn_fs_root_t *root,
 
       if (! allowed)
         return svn_error_create (SVN_ERR_AUTHZ_ROOT_UNREADABLE, 0,
-                                 "Unable to open root of edit");
+                                 _("Unable to open root of edit"));
     }
 
   return SVN_NO_ERROR;
@@ -216,7 +219,6 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
   svn_node_kind_t src_kind, tgt_kind;
   svn_revnum_t rootrev;
   int distance;
-  svn_boolean_t allowed = TRUE;  /* for authz checks */
   const char *authz_root_path;
 
   /* SRC_PARENT_DIR must be valid. */
@@ -226,7 +228,7 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
   /* TGT_FULLPATH must be valid. */
   if (! tgt_fullpath)
     return svn_error_create (SVN_ERR_FS_PATH_SYNTAX, 0,
-                             "Invalid target path");
+                             _("Invalid target path"));
 
   /* Calculate the fs path implicitly used for editor->open_root, so
      we can do an authz check on that path first. */
@@ -252,8 +254,8 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
                          || tgt_kind != svn_node_dir))
     return svn_error_create 
       (SVN_ERR_FS_PATH_SYNTAX, 0,
-       "Invalid editor anchoring; at least one of the "
-       "input paths is not a directory and there was no source entry");
+       _("Invalid editor anchoring; at least one of the "
+         "input paths is not a directory and there was no source entry"));
   
   /* Set the global target revision if one can be determined. */
   if (svn_fs_is_revision_root (tgt_root))
@@ -307,24 +309,8 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
       SVN_ERR (authz_root_check (tgt_root, authz_root_path,
                                  authz_read_func, authz_read_baton, pool));
       SVN_ERR (editor->open_root (edit_baton, rootrev, pool, &root_baton));
-
-      if (authz_read_func)
-        {
-          SVN_ERR (authz_read_func (&allowed, tgt_root, tgt_fullpath,
-                                    authz_read_baton, pool));
-        }
-      
-      if (allowed)
-        {
-          SVN_ERR (add_file_or_dir (&c, root_baton, tgt_fullpath,
-                                    src_entry, tgt_kind, pool));
-        }
-      else
-        {
-          SVN_ERR (absent_file_or_dir (&c, root_baton,
-                                       src_entry, tgt_kind, pool));
-        }
-
+      SVN_ERR (add_file_or_dir (&c, root_baton, tgt_fullpath,
+                                src_entry, tgt_kind, pool));
       goto cleanup;
     }
 
@@ -349,24 +335,9 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
           SVN_ERR (authz_root_check (tgt_root, authz_root_path,
                                      authz_read_func, authz_read_baton, pool));
           SVN_ERR (editor->open_root (edit_baton, rootrev, pool, &root_baton));
-          SVN_ERR (delete (&c, root_baton, src_fullpath, pool));
-
-          if (authz_read_func)
-            {
-              SVN_ERR (authz_read_func (&allowed, tgt_root, tgt_fullpath,
-                                        authz_read_baton, pool));
-            }
-          
-          if (allowed)
-            {
-              SVN_ERR (add_file_or_dir (&c, root_baton, tgt_fullpath, 
-                                        src_entry, tgt_kind, pool));
-            }
-          else
-            {
-              SVN_ERR (absent_file_or_dir (&c, root_baton,
-                                           src_entry, tgt_kind, pool));
-            }
+          SVN_ERR (delete (&c, root_baton, src_entry, pool));
+          SVN_ERR (add_file_or_dir (&c, root_baton, tgt_fullpath, 
+                                    src_entry, tgt_kind, pool));
         }
       /* Otherwise, we just replace the one with the other. */
       else
@@ -374,24 +345,9 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
           SVN_ERR (authz_root_check (tgt_root, authz_root_path,
                                      authz_read_func, authz_read_baton, pool));
           SVN_ERR (editor->open_root (edit_baton, rootrev, pool, &root_baton));
-
-          if (authz_read_func)
-            {
-              SVN_ERR (authz_read_func (&allowed, tgt_root, tgt_fullpath,
-                                        authz_read_baton, pool));
-            }
-
-          if (allowed)
-            {
-              SVN_ERR (replace_file_or_dir (&c, root_baton, src_fullpath,
-                                            tgt_fullpath, src_entry, 
-                                            tgt_kind, pool));
-            }
-          else
-            {
-              SVN_ERR (absent_file_or_dir (&c, root_baton,
-                                           src_entry, tgt_kind, pool));
-            }
+          SVN_ERR (replace_file_or_dir (&c, root_baton, src_fullpath,
+                                        tgt_fullpath, src_entry, 
+                                        tgt_kind, pool));
         }
     }
   else
@@ -524,22 +480,26 @@ delta_proplists (struct context *c,
       svn_string_t *last_author = NULL;
       
       /* Get the CR and two derivative props. ### check for error returns. */
-      svn_fs_node_created_rev (&committed_rev, c->target_root, 
-                               target_path, subpool);
+      SVN_ERR (svn_fs_node_created_rev (&committed_rev, c->target_root,
+                                        target_path, subpool));
       if (SVN_IS_VALID_REVNUM (committed_rev))
         {
           svn_fs_t *fs = svn_fs_root_fs (c->target_root);
+          apr_hash_t *r_props;
           const char *uuid;
 
           /* Transmit the committed-rev. */
-          cr_str = svn_string_createf (subpool, "%" SVN_REVNUM_T_FMT,
+          cr_str = svn_string_createf (subpool, "%ld",
                                        committed_rev);
           SVN_ERR (change_fn (c, object, SVN_PROP_ENTRY_COMMITTED_REV, 
                               cr_str, subpool));
 
+          SVN_ERR (svn_fs_revision_proplist (&r_props, fs, committed_rev,
+                                             pool));
+
           /* Transmit the committed-date. */
-          svn_fs_revision_prop (&committed_date, fs, committed_rev, 
-                                SVN_PROP_REVISION_DATE, subpool);
+          committed_date = apr_hash_get (r_props, SVN_PROP_REVISION_DATE,
+                                         APR_HASH_KEY_STRING);
           if (committed_date || source_path)
             {
               SVN_ERR (change_fn (c, object, SVN_PROP_ENTRY_COMMITTED_DATE, 
@@ -547,8 +507,8 @@ delta_proplists (struct context *c,
             }
 
           /* Transmit the last-author. */
-          svn_fs_revision_prop (&last_author, fs, committed_rev, 
-                                SVN_PROP_REVISION_AUTHOR, subpool);
+          last_author = apr_hash_get (r_props, SVN_PROP_REVISION_AUTHOR,
+                                      APR_HASH_KEY_STRING);
           if (last_author || source_path)
             {
               SVN_ERR (change_fn (c, object, SVN_PROP_ENTRY_LAST_AUTHOR,
@@ -556,7 +516,7 @@ delta_proplists (struct context *c,
             }
 
           /* Transmit the UUID. */
-          svn_fs_get_uuid (fs, &uuid, subpool);
+          SVN_ERR (svn_fs_get_uuid (fs, &uuid, subpool));
           if (uuid || source_path)
             {
               SVN_ERR (change_fn (c, object, SVN_PROP_ENTRY_UUID,
@@ -657,7 +617,7 @@ compare_files (svn_boolean_t *changed_p,
   svn_filesize_t size1, size2;
   unsigned char digest1[APR_MD5_DIGESTSIZE], digest2[APR_MD5_DIGESTSIZE];
   svn_stream_t *stream1, *stream2;
-  char buf1[SVN_STREAM_CHUNK_SIZE], buf2[SVN_STREAM_CHUNK_SIZE];
+  char *buf1, *buf2;
   apr_size_t len1, len2;
 
   /* If the filesystem claims the things haven't changed, then they
@@ -696,6 +656,8 @@ compare_files (svn_boolean_t *changed_p,
   SVN_ERR (svn_fs_file_contents (&stream1, root1, path1, pool));
   SVN_ERR (svn_fs_file_contents (&stream2, root2, path2, pool));
 
+  buf1 = apr_palloc (pool, SVN_STREAM_CHUNK_SIZE);
+  buf2 = apr_palloc (pool, SVN_STREAM_CHUNK_SIZE);
   do
     {
       len1 = len2 = SVN_STREAM_CHUNK_SIZE;
@@ -817,9 +779,10 @@ delete (struct context *c,
 }
 
 
-/* Emit a delta to create the entry named TARGET_ENTRY in the
-   directory TARGET_PARENT.  Pass DIR_BATON through to editor
-   functions that require it. */
+/* If authorized, emit a delta to create the entry named TARGET_ENTRY
+   at the location EDIT_PATH.  If not authorized, indicate that
+   EDIT_PATH is absent.  Pass DIR_BATON through to editor functions
+   that require it. */
 static svn_error_t *
 add_file_or_dir (struct context *c, void *dir_baton,
                  const char *target_path,
@@ -828,10 +791,19 @@ add_file_or_dir (struct context *c, void *dir_baton,
                  apr_pool_t *pool)
 {
   struct context *context = c;
+  svn_boolean_t allowed;
 
   /* Sanity-check our input. */
   assert (target_path && edit_path);
 
+  if (c->authz_read_func)
+    {
+      SVN_ERR (c->authz_read_func (&allowed, c->target_root, target_path,
+                                   c->authz_read_baton, pool));
+      if (!allowed)
+        return absent_file_or_dir (c, dir_baton, edit_path, tgt_kind, pool);
+    }
+
   if (tgt_kind == svn_node_dir)
     {
       void *subdir_baton;
@@ -862,9 +834,10 @@ add_file_or_dir (struct context *c, void *dir_baton,
 }
 
 
-/* Modify the directory TARGET_PARENT by replacing its entry named
-   TARGET_ENTRY with the SOURCE_ENTRY found in SOURCE_PARENT.  Pass
-   DIR_BATON through to editor functions that require it. */
+/* If authorized, emit a delta to modify EDIT_PATH with the changes
+   from SOURCE_PATH to TARGET_PATH.  If not authorized, indicate that
+   EDIT_PATH is absent.  Pass DIR_BATON through to editor functions
+   that require it. */
 static svn_error_t *
 replace_file_or_dir (struct context *c, 
                      void *dir_baton,
@@ -875,10 +848,19 @@ replace_file_or_dir (struct context *c,
                      apr_pool_t *pool)
 {
   svn_revnum_t base_revision = SVN_INVALID_REVNUM;
+  svn_boolean_t allowed;
 
   /* Sanity-check our input. */
   assert (target_path && source_path && edit_path);
 
+  if (c->authz_read_func)
+    {
+      SVN_ERR (c->authz_read_func (&allowed, c->target_root, target_path,
+                                   c->authz_read_baton, pool));
+      if (!allowed)
+        return absent_file_or_dir (c, dir_baton, edit_path, tgt_kind, pool);
+    }
+
   /* Get the base revision for the entry from the hash. */
   base_revision = get_path_revision (c->source_root, source_path, pool);
 
@@ -946,7 +928,6 @@ delta_dirs (struct context *c,
   apr_hash_t *s_entries = 0, *t_entries = 0;
   apr_hash_index_t *hi;
   apr_pool_t *subpool;
-  svn_boolean_t allowed = TRUE;
 
   assert (target_path);
 
@@ -985,6 +966,9 @@ delta_dirs (struct context *c,
       const char *s_fullpath;
       svn_node_kind_t tgt_kind;
 
+      /* Clear out our subpool for the next iteration... */
+      svn_pool_clear (subpool);
+
       /* KEY is the entry name in target, VAL the dirent */
       apr_hash_this (hi, &key, &klen, &val);
       t_entry = val;
@@ -1021,47 +1005,14 @@ delta_dirs (struct context *c,
                        || ((distance == -1) && (! c->ignore_ancestry)))
                 {
                   SVN_ERR (delete (c, dir_baton, e_fullpath, subpool));
-
-                  if (c->authz_read_func)
-                    {
-                      SVN_ERR (c->authz_read_func
-                               (&allowed, c->target_root, t_fullpath,
-                                c->authz_read_baton, subpool));
-                    }
-
-                  if (allowed)
-                    {
-                      SVN_ERR (add_file_or_dir (c, dir_baton, t_fullpath,
-                                                e_fullpath, tgt_kind,
-                                                subpool));
-                    }
-                  else
-                    {
-                      SVN_ERR (absent_file_or_dir (c, dir_baton, e_fullpath,
-                                                   tgt_kind, subpool));
-                    }
+                  SVN_ERR (add_file_or_dir (c, dir_baton, t_fullpath,
+                                            e_fullpath, tgt_kind, subpool));
                 }
               else
                 {
-                  if (c->authz_read_func)
-                    {
-                      SVN_ERR (c->authz_read_func
-                               (&allowed,
-                                c->target_root, t_fullpath,
-                                c->authz_read_baton, subpool));
-                    }
-
-                  if (allowed)
-                    {
-                      SVN_ERR (replace_file_or_dir (c, dir_baton, s_fullpath,
-                                                    t_fullpath, e_fullpath, 
-                                                    tgt_kind, subpool));
-                    }
-                  else
-                    {
-                      SVN_ERR (absent_file_or_dir (c, dir_baton, e_fullpath,
-                                                   tgt_kind, subpool));
-                    }
+                  SVN_ERR (replace_file_or_dir (c, dir_baton, s_fullpath,
+                                                t_fullpath, e_fullpath, 
+                                                tgt_kind, subpool));
                 }
             }
 
@@ -1072,32 +1023,10 @@ delta_dirs (struct context *c,
         {
           if (c->recurse || (tgt_kind != svn_node_dir))
             {
-              /* We didn't find an entry with this name in the source
-                 entries hash.  This must be something new that needs to
-                 be added. */
-              if (c->authz_read_func)
-                {
-                  SVN_ERR (c->authz_read_func
-                           (&allowed,
-                            c->target_root, t_fullpath,
-                            c->authz_read_baton, subpool));
-                }
-
-              if (allowed)
-                {
-                  SVN_ERR (add_file_or_dir (c, dir_baton, t_fullpath,
-                                            e_fullpath, tgt_kind, subpool));
-                }
-              else
-                {
-                  SVN_ERR (absent_file_or_dir (c, dir_baton, e_fullpath,
-                                               tgt_kind, subpool));
-                }
+              SVN_ERR (add_file_or_dir (c, dir_baton, t_fullpath,
+                                        e_fullpath, tgt_kind, subpool));
             }
         }
-
-      /* Clear out our subpool for the next iteration... */
-      svn_pool_clear (subpool);
     }
 
   /* All that is left in the source entries hash are things that need
@@ -1110,22 +1039,21 @@ delta_dirs (struct context *c,
           const void *key;
           void *val;
           apr_ssize_t klen;
-          const char *s_fullpath, *e_fullpath;
+          const char *e_fullpath;
           svn_node_kind_t src_kind;
           
+          /* Clear out our subpool for the next iteration... */
+          svn_pool_clear (subpool);
+
           /* KEY is the entry name in source, VAL the dirent */
           apr_hash_this (hi, &key, &klen, &val);
           s_entry = val;
           src_kind = s_entry->kind;
-          s_fullpath = svn_path_join (source_path, s_entry->name, subpool);
           e_fullpath = svn_path_join (edit_path, s_entry->name, subpool);
 
           /* Do we actually want to delete the dir if we're non-recursive? */
           if (c->recurse || (src_kind != svn_node_dir))
             SVN_ERR (delete (c, dir_baton, e_fullpath, subpool));
-
-          /* Clear out our subpool for the next iteration... */
-          svn_pool_clear (subpool);
         }
     }
 
