@@ -27,10 +27,8 @@
 #include <apr_tables.h>
 #include <apr_file_io.h>
 #include <apr_strings.h>
-#include <apr_lib.h>
 #include <apr_general.h>
 #include "svn_types.h"
-#include "svn_delta.h"
 #include "svn_string.h"
 #include "svn_pools.h"
 #include "svn_path.h"
@@ -40,7 +38,6 @@
 #include "svn_io.h"
 #include "svn_hash.h"
 #include "svn_wc.h"
-#include "svn_time.h"
 #include "svn_utf.h"
 
 #include "wc.h"
@@ -51,6 +48,7 @@
 #include "translate.h"
 #include "questions.h"
 
+#include "svn_private_config.h"
 
 /*---------------------------------------------------------------------*/
 
@@ -99,7 +97,7 @@ svn_wc__conflicting_propchanges_p (const svn_string_t **description,
       *description =
         svn_string_createf
         (pool,
-         "Property '%s' locally changed to '%s', but update deletes it\n",
+         _("Property '%s' locally changed to '%s', but update deletes it\n"),
          local->name, local->value->data);
       return TRUE;  /* conflict */
     }
@@ -107,7 +105,8 @@ svn_wc__conflicting_propchanges_p (const svn_string_t **description,
     {
       *description =
         svn_string_createf
-        (pool, "Property '%s' locally deleted, but update sets it to '%s'\n",
+        (pool,
+         _("Property '%s' locally deleted, but update sets it to '%s'\n"),
          local->name, update->value->data);
       return TRUE;  /* conflict */
     }
@@ -123,8 +122,8 @@ svn_wc__conflicting_propchanges_p (const svn_string_t **description,
     {
       *description =
         svn_string_createf
-        (pool, "Property '%s' locally changed to '%s', "
-         "but update sets it to '%s'\n",
+        (pool, _("Property '%s' locally changed to '%s', "
+                 "but update sets it to '%s'\n"),
          local->name, local->value->data, update->value->data);
       return TRUE;  /* conflict */
     }
@@ -166,7 +165,8 @@ svn_wc__load_prop_file (const char *propfile_path,
                                  pool));
 
       SVN_ERR_W (svn_hash_read (hash, propfile, pool),
-                 apr_psprintf (pool, "Can't parse '%s'", propfile_path));
+                 apr_psprintf (pool, _("Can't parse '%s'"),
+                               svn_path_local_style (propfile_path, pool)));
 
       SVN_ERR (svn_io_file_close (propfile, pool));
     }
@@ -192,8 +192,8 @@ svn_wc__save_prop_file (const char *propfile_path,
 
   SVN_ERR_W (svn_hash_write (hash, prop_tmp, pool),
              apr_psprintf (pool, 
-                           "Can't write property hash to '%s'", 
-                           propfile_path));
+                           _("Can't write property hash to '%s'"),
+                           svn_path_local_style (propfile_path, pool)));
 
   SVN_ERR (svn_io_file_close (prop_tmp, pool));
 
@@ -245,8 +245,9 @@ svn_wc__get_existing_prop_reject_file (const char **reject_file,
   if (! the_entry)
     return svn_error_createf
       (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-       "Can't find entry '%s' in '%s'", 
-       name, svn_wc_adm_access_path (adm_access));
+       _("Can't find entry '%s' in '%s'"),
+       name,
+       svn_path_local_style (svn_wc_adm_access_path (adm_access), pool));
 
   *reject_file = the_entry->prejfile 
                  ? apr_pstrdup (pool, the_entry->prejfile)
@@ -277,7 +278,8 @@ svn_wc_merge_prop_diffs (svn_wc_notify_state_t *state,
   SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
   if (entry == NULL)
     return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                              "'%s' is not under version control", path);
+                              _("'%s' is not under version control"),
+                              svn_path_local_style (path, pool));
 
   /* Notice that we're not using svn_path_split_if_file(), because
      that looks at the actual working file.  It's existence shouldn't
@@ -313,7 +315,8 @@ svn_wc_merge_prop_diffs (svn_wc_notify_state_t *state,
     {
       SVN_ERR_W (svn_io_file_write_full (log_fp, log_accum->data, 
                                          log_accum->len, NULL, pool),
-                 apr_psprintf (pool, "Error writing log for '%s'", path));
+                 apr_psprintf (pool, _("Error writing log for '%s'"),
+                               svn_path_local_style (path, pool)));
 
       SVN_ERR (svn_wc__close_adm_file (log_fp, parent, SVN_WC__ADM_LOG,
                                        1, /* sync */ pool));
@@ -345,7 +348,7 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
 
   const char *access_path = svn_wc_adm_access_path (adm_access);
   int access_len = strlen (access_path);
-  int slash = 1;
+  int slash;
 
   const char *entryname;
   const char *full_path;
@@ -365,6 +368,8 @@ svn_wc__merge_prop_diffs (svn_wc_notify_state_t *state,
   /* Empty path and paths ending in / don't need an extra slash removed */
   if (access_len == 0 || access_path[access_len - 1] == '/')
     slash = 0;
+  else
+    slash = 1;
 
   if (name == NULL)
     {
@@ -729,13 +734,13 @@ wcprop_list (apr_hash_t **props,
 #if 0
   if (kind == svn_node_none)
     return svn_error_createf (SVN_ERR_BAD_FILENAME, NULL,
-                              "'%s' does not exist",
-                              path);
+                              _("'%s' does not exist"),
+                              svn_path_local_style (path, pool));
   
   if (kind == svn_node_unknown)
     return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                              "Unknown node kind: '%s'",
-                              path);
+                              _("Unknown node kind: '%s'"),
+                              svn_path_local_style (path, pool));
 #endif
 
   /* Construct a path to the relevant property file */
@@ -770,7 +775,7 @@ svn_wc__wcprop_get (const svn_string_t **value,
   if (err)
     return
       svn_error_quick_wrap
-      (err, "Failed to load properties from disk");
+      (err, _("Failed to load properties from disk"));
 
   *value = apr_hash_get (prophash, name, APR_HASH_KEY_STRING);
   return SVN_NO_ERROR;
@@ -792,7 +797,7 @@ svn_wc__wcprop_set (const char *name,
   if (err)
     return
       svn_error_quick_wrap
-      (err, "Failed to load properties from disk");
+      (err, _("Failed to load properties from disk"));
 
   /* Now we have all the properties in our hash.  Simply merge the new
      property into it. */
@@ -808,7 +813,8 @@ svn_wc__wcprop_set (const char *name,
   /* Write. */
   SVN_ERR_W (svn_hash_write (prophash, fp, pool),
              apr_psprintf (pool,
-                           "Cannot write property hash for '%s'", path));
+                           _("Cannot write property hash for '%s'"),
+                           svn_path_local_style (path, pool)));
   
   /* Close file, and doing an atomic "move". */
   SVN_ERR (svn_wc__close_props (fp, path, 0, 1,
@@ -832,7 +838,6 @@ svn_wc_prop_list (apr_hash_t **props,
                   svn_wc_adm_access_t *adm_access,
                   apr_pool_t *pool)
 {
-  svn_node_kind_t pkind;
   const char *prop_path;
 
   *props = apr_hash_make (pool);
@@ -840,18 +845,8 @@ svn_wc_prop_list (apr_hash_t **props,
   /* Construct a path to the relevant property file */
   SVN_ERR (svn_wc__prop_path (&prop_path, path, adm_access, FALSE, pool));
 
-  /* Does the property file exist? */
-  SVN_ERR (svn_io_check_path (prop_path, &pkind, pool));
-  
-  if (pkind == svn_node_none)
-    /* No property file exists.  Just go home, with an empty hash. */
-    return SVN_NO_ERROR;
-  
-  /* else... */
-
-  SVN_ERR (svn_wc__load_prop_file (prop_path, *props, pool));
-
-  return SVN_NO_ERROR;
+  /* svn_wc__load_prop_file checks if the prop file exists */
+  return svn_wc__load_prop_file (prop_path, *props, pool);
 }
 
 
@@ -878,7 +873,7 @@ svn_wc_prop_get (const svn_string_t **value,
     {
       return svn_error_createf   /* we don't do entry properties here */
         (SVN_ERR_BAD_PROP_KIND, NULL,
-         "Property '%s' is an entry property", name);
+         _("Property '%s' is an entry property"), name);
     }
   else  /* regular prop */
     {
@@ -886,7 +881,7 @@ svn_wc_prop_get (const svn_string_t **value,
       if (err)
         return
           svn_error_quick_wrap
-          (err, "Failed to load properties from disk");
+          (err, _("Failed to load properties from disk"));
       
       *value = apr_hash_get (prophash, name, APR_HASH_KEY_STRING);
 
@@ -912,31 +907,35 @@ validate_prop_against_node_kind (const char *name,
                                  SVN_PROP_KEYWORDS,
                                  SVN_PROP_EOL_STYLE,
                                  SVN_PROP_MIME_TYPE,
+                                 SVN_PROP_NEEDS_LOCK,
                                  NULL };
   const char **node_kind_prohibit;
-  const char *node_kind_text;
 
   switch (node_kind)
     {
     case svn_node_dir:
       node_kind_prohibit = dir_prohibit;
-      node_kind_text = "directory";
+      while (*node_kind_prohibit)
+        if (strcmp (name, *node_kind_prohibit++) == 0)
+          return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
+                                    _("Cannot set '%s' on a directory ('%s')"),
+                                    name, svn_path_local_style (path, pool));
       break;
     case svn_node_file:
       node_kind_prohibit = file_prohibit;
-      node_kind_text = "file";
+      while (*node_kind_prohibit)
+        if (strcmp (name, *node_kind_prohibit++) == 0)
+          return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
+                                    _("Cannot set '%s' on a file ('%s')"),
+                                    name,
+                                    svn_path_local_style (path, pool));
       break;
     default:
       return svn_error_createf (SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
-                                "'%s' is not a file or directory", path);
+                                _("'%s' is not a file or directory"),
+                                svn_path_local_style (path, pool));
     }
 
-  while (*node_kind_prohibit)
-    if (strcmp (name, *node_kind_prohibit++) == 0)
-      return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
-                                "Cannot set '%s' on a %s (%s)",
-                                name, node_kind_text, path);
-
   return SVN_NO_ERROR;
 }                             
 
@@ -955,8 +954,10 @@ validate_eol_prop_against_file (const char *path,
   SVN_ERR (svn_wc_prop_get (&mime_type, SVN_PROP_MIME_TYPE, path, adm_access,
                             pool));
   if (mime_type && svn_mime_type_is_binary (mime_type->data))
-    return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, NULL,
-                              "File '%s' has binary mime type property", path);
+    return svn_error_createf 
+      (SVN_ERR_ILLEGAL_TARGET, NULL,
+       _("File '%s' has binary mime type property"),
+       svn_path_local_style (path, pool));
 
   /* Open PATH. */
   SVN_ERR (svn_io_file_open (&fp, path, 
@@ -973,11 +974,12 @@ validate_eol_prop_against_file (const char *path,
      here is whether or not the function fails on inconsistent line
      endings.  The function is "translating" to an empty stream.  This
      is sneeeeeeeeeeeaky. */
-  err = svn_subst_translate_stream (read_stream, write_stream, 
-                                    "", FALSE, NULL, FALSE);
+  err = svn_subst_translate_stream2 (read_stream, write_stream, 
+                                     "", FALSE, NULL, FALSE, pool);
   if (err && err->apr_err == SVN_ERR_IO_INCONSISTENT_EOL)
     return svn_error_createf (SVN_ERR_ILLEGAL_TARGET, err,
-                              "File '%s' has inconsistent newlines", path);
+                              _("File '%s' has inconsistent newlines"),
+                              svn_path_local_style (path, pool));
   else if (err)
     return err;
 
@@ -987,11 +989,12 @@ validate_eol_prop_against_file (const char *path,
 
 
 svn_error_t *
-svn_wc_prop_set (const char *name,
-                 const svn_string_t *value,
-                 const char *path,
-                 svn_wc_adm_access_t *adm_access,
-                 apr_pool_t *pool)
+svn_wc_prop_set2 (const char *name,
+                  const svn_string_t *value,
+                  const char *path,
+                  svn_wc_adm_access_t *adm_access,
+                  svn_boolean_t skip_checks,
+                  apr_pool_t *pool)
 {
   svn_error_t *err;
   apr_hash_t *prophash;
@@ -1008,23 +1011,25 @@ svn_wc_prop_set (const char *name,
   else if (prop_kind == svn_prop_entry_kind)
     return svn_error_createf   /* we don't do entry properties here */
       (SVN_ERR_BAD_PROP_KIND, NULL,
-       "Property '%s' is an entry property", name);
+       _("Property '%s' is an entry property"), name);
 
   /* Else, handle a regular property: */
 
-  /* Setting an inappropriate property is not allowed, deleting such a
-     property is allowed since older clients allowed (and other clients
-     possibly still allow) setting it. */
+  /* Setting an inappropriate property is not allowed (unless
+     overridden by 'skip_checks', in some circumstances).  Deleting an
+     inappropriate property is allowed, however, since older clients
+     allowed (and other clients possibly still allow) setting it in
+     the first place. */
   if (value)
     {
       SVN_ERR (validate_prop_against_node_kind (name, path, kind, pool));
-      if (strcmp (name, SVN_PROP_EOL_STYLE) == 0)
+      if (!skip_checks && (strcmp (name, SVN_PROP_EOL_STYLE) == 0))
         {
           new_value = svn_stringbuf_create_from_string (value, pool);
           svn_stringbuf_strip_whitespace (new_value);
           SVN_ERR (validate_eol_prop_against_file (path, adm_access, pool));
         }
-      else if (strcmp (name, SVN_PROP_MIME_TYPE) == 0)
+      else if (!skip_checks && (strcmp (name, SVN_PROP_MIME_TYPE) == 0))
         {
           new_value = svn_stringbuf_create_from_string (value, pool);
           svn_stringbuf_strip_whitespace (new_value);
@@ -1040,7 +1045,10 @@ svn_wc_prop_set (const char *name,
               svn_stringbuf_appendbytes (new_value, "\n", 1);
             }
 
-          /* Make sure this is a valid externals property. */
+          /* Make sure this is a valid externals property.  Do not
+             allow 'skip_checks' to override, as there is no circumstance in
+             which this is proper (because there is no circumstance in
+             which Subversion can handle it). */
           if (strcmp (name, SVN_PROP_EXTERNALS) == 0)
             {
               /* We don't allow "." nor ".." as target directories in
@@ -1048,7 +1056,7 @@ svn_wc_prop_set (const char *name,
                  checks for this, so all we have to is invoke it --
                  we're not interested in the parsed result, only in
                  whether or the parsing errored. */
-              SVN_ERR (svn_wc_parse_externals_description
+              SVN_ERR (svn_wc_parse_externals_description2
                        (NULL, path, value->data, pool));
             }
         }
@@ -1083,11 +1091,35 @@ svn_wc_prop_set (const char *name,
         }
     }
 
+  if (kind == svn_node_file && strcmp (name, SVN_PROP_NEEDS_LOCK) == 0)
+    {
+      /* If the svn:needs-lock property was set to NULL, set the file
+         to read-write */
+      if (value == NULL)
+        {
+          SVN_ERR (svn_io_set_file_read_write_carefully (path, TRUE, 
+                                                         FALSE, pool));
+        }
+      else
+        {
+          /* Since we only check if the property exists or not, force the
+             property value to a specific value */
+          static const svn_string_t needs_lock_value =
+            {
+              SVN_PROP_NEEDS_LOCK_VALUE,
+              sizeof (SVN_PROP_NEEDS_LOCK_VALUE) - 1
+            };
+
+          value = &needs_lock_value;
+          /* And we'll set the file to read-only at commit time. */
+        }
+    }
+
   err = svn_wc_prop_list (&prophash, path, adm_access, pool);
   if (err)
     return
       svn_error_quick_wrap
-      (err, "Failed to load properties from disk");
+      (err, _("Failed to load properties from disk"));
 
   /* If we're changing this file's list of expanded keywords, then
    * we'll need to invalidate its text timestamp, since keyword
@@ -1117,7 +1149,8 @@ svn_wc_prop_set (const char *name,
   /* Write. */
   SVN_ERR_W (svn_hash_write (prophash, fp, pool),
              apr_psprintf (pool, 
-                           "Cannot write property hash for '%s'", path));
+                           _("Cannot write property hash for '%s'"),
+                           svn_path_local_style (path, pool)));
   
   /* Close file, and doing an atomic "move". */
   SVN_ERR (svn_wc__close_props (fp, path, 0, 0,
@@ -1151,6 +1184,18 @@ svn_wc_prop_set (const char *name,
 }
 
 
+svn_error_t *
+svn_wc_prop_set (const char *name,
+                 const svn_string_t *value,
+                 const char *path,
+                 svn_wc_adm_access_t *adm_access,
+                 apr_pool_t *pool)
+{
+  return svn_wc_prop_set2 (name, value, path, adm_access, FALSE, pool);
+}
+
+
+
 svn_boolean_t
 svn_wc_is_normal_prop (const char *name)
 {
@@ -1185,22 +1230,28 @@ empty_props_p (svn_boolean_t *empty_p,
                const char *path_to_prop_file,
                apr_pool_t *pool)
 {
-  svn_node_kind_t kind;
+  svn_error_t *err;
+  apr_finfo_t finfo;
 
-  SVN_ERR (svn_io_check_path (path_to_prop_file, &kind, pool));
+  err = svn_io_stat (&finfo, path_to_prop_file, APR_FINFO_MIN | APR_FINFO_TYPE,
+                     pool);
+  if (err)
+    {
+      if (! APR_STATUS_IS_ENOENT (err->apr_err)
+          && ! APR_STATUS_IS_ENOTDIR (err->apr_err))
+        return err;
 
-  if (kind == svn_node_none)
-    *empty_p = TRUE;
-
-  else 
+      /* nonexistent */
+      svn_error_clear (err);
+      *empty_p = TRUE;
+    }
+  else
     {
-      apr_finfo_t finfo;
 
-      SVN_ERR (svn_io_stat (&finfo, path_to_prop_file, APR_FINFO_MIN, pool));
 
       /* If we remove props from a propfile, eventually the file will
          contain nothing but "END\n" */
-      if (finfo.size == 4)  
+      if (finfo.filetype == APR_REG && finfo.size == 4)  
         *empty_p = TRUE;
 
       else
@@ -1303,7 +1354,7 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
   
   /* There are at least three tests we can try in succession. */
   
-  /* Easy-answer attempt #1:  */
+  /* Easy-answer attempt #1:  (### this stat's the files again) */
   
   /* Check if the local and prop-base file have *definitely* different
      filesizes. */
@@ -1317,7 +1368,7 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
       goto cleanup;
     }
   
-  /* Easy-answer attempt #2:  */
+  /* Easy-answer attempt #2:  (### this stat's the files again) */
       
   /* See if the local file's prop timestamp is the same as the one
      recorded in the administrative directory.  */
@@ -1348,6 +1399,7 @@ svn_wc_props_modified_p (svn_boolean_t *modified_p,
     apr_hash_t *localprops = apr_hash_make (subpool);
     apr_hash_t *baseprops = apr_hash_make (subpool);
 
+    /* ### Amazingly, this stats the files again! */
     SVN_ERR (svn_wc__load_prop_file (prop_path, localprops, subpool));
     SVN_ERR (svn_wc__load_prop_file (prop_base_path,
                                      baseprops,
@@ -1434,16 +1486,16 @@ svn_wc_get_prop_diffs (apr_array_header_t **propchanges,
 /** Externals **/
 
 svn_error_t *
-svn_wc_parse_externals_description (apr_hash_t **externals_p,
-                                    const char *parent_directory,
-                                    const char *desc,
-                                    apr_pool_t *pool)
+svn_wc_parse_externals_description2 (apr_array_header_t **externals_p,
+                                     const char *parent_directory,
+                                     const char *desc,
+                                     apr_pool_t *pool)
 {
   apr_array_header_t *lines = svn_cstring_split (desc, "\n\r", TRUE, pool);
   int i;
   
   if (externals_p)
-    *externals_p = apr_hash_make (pool);
+    *externals_p = apr_array_make (pool, 1, sizeof (svn_wc_external_item_t *));
 
   for (i = 0; i < lines->nelts; i++)
     {
@@ -1523,35 +1575,59 @@ svn_wc_parse_externals_description (apr_hash_t **externals_p,
         parse_error:
           return svn_error_createf
             (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION, NULL,
-             "Error parsing " SVN_PROP_EXTERNALS " property on '%s': '%s'",
-             parent_directory, line);
+             _("Error parsing %s property on '%s': '%s'"),
+             SVN_PROP_EXTERNALS,
+             svn_path_local_style (parent_directory, pool),
+             line);
         }
 
-      /* Make sure we don't have "." nor "../" in the tgt dir. */
+      item->target_dir = svn_path_canonicalize
+        (svn_path_internal_style (item->target_dir, pool), pool);
       {
-        int tgt_dir_len = strlen (item->target_dir);
-
-        if ((strcmp (item->target_dir, ".") == 0)
-            || (strcmp (item->target_dir, "./") == 0)
-            || (strncmp (item->target_dir, "../", 3) == 0)
-            || (strstr (item->target_dir, "/../") != NULL)
-            || (strncmp ((item->target_dir + tgt_dir_len - 3), "/..", 3) == 0))
+        if (item->target_dir[0] == '\0' || item->target_dir[0] == '/'
+            || svn_path_is_backpath_present (item->target_dir))
           return svn_error_createf
             (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION, NULL,
-             "Invalid " SVN_PROP_EXTERNALS " property on '%s': "
-             "target involves '.' or '..'",
-             parent_directory);
+             _("Invalid %s property on '%s': "
+               "target involves '.' or '..' or is an absolute path"),
+             SVN_PROP_EXTERNALS,
+             svn_path_local_style (parent_directory, pool));
       }
 
-      item->target_dir = svn_path_canonicalize (item->target_dir, pool);
       item->url = svn_path_canonicalize (item->url, pool);
 
       if (externals_p)
-        {
-          apr_hash_set (*externals_p, item->target_dir,
-                        APR_HASH_KEY_STRING, item);
-        }
+        APR_ARRAY_PUSH (*externals_p, svn_wc_external_item_t *) = item;
     }
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_wc_parse_externals_description (apr_hash_t **externals_p,
+                                    const char *parent_directory,
+                                    const char *desc,
+                                    apr_pool_t *pool)
+{
+  apr_array_header_t *list;
+  int i;
+
+  SVN_ERR (svn_wc_parse_externals_description2 (externals_p ? &list : NULL,
+                                                parent_directory, desc, pool));
+
+  /* Store all of the items into the hash if that was requested. */
+  if (externals_p)
+    {
+      *externals_p = apr_hash_make (pool);
+      for (i = 0; i < list->nelts; i++)
+        {
+          svn_wc_external_item_t *item;
+          item = APR_ARRAY_IDX (list, i, svn_wc_external_item_t *);
+
+          apr_hash_set (*externals_p, item->target_dir,
+                        APR_HASH_KEY_STRING, item);
+        }
+    }
+  return SVN_NO_ERROR;
+}
