@@ -29,9 +29,8 @@
 #include "svn_pools.h"
 #include "svn_dav.h"
 #include "svn_base64.h"
+#include "svn_props.h"
 
-/* Private header included just for 1.0.7 revprops security fix. */
-#include "../libsvn_repos/repos.h"
 
 struct dav_db {
   const dav_resource *resource;
@@ -108,12 +107,11 @@ static dav_error *get_value(dav_db *db, const dav_prop_name *name,
       serr = svn_fs_txn_prop(pvalue, db->resource->info->root.txn,
                              propname, db->p);
     else
-      serr = svn_repos__fs_revision_prop
-        (pvalue,
-         db->resource->info-> repos->repos,
-         db->resource->info->root.rev, propname,
-         db->authz_read_func,
-         db->authz_read_baton, db->p);
+      serr = svn_repos_fs_revision_prop(pvalue,
+                                        db->resource->info-> repos->repos,
+                                        db->resource->info->root.rev, propname,
+                                        db->authz_read_func,
+                                        db->authz_read_baton, db->p);
   else
     serr = svn_fs_node_prop(pvalue, db->resource->info->root.root,
                             get_repos_path(db->resource->info),
@@ -157,14 +155,13 @@ static dav_error *save_value(dav_db *db, const dav_prop_name *name,
          not a working resource!  But this is how we currently
          (hackily) allow the svn client to change unversioned rev
          props.  See issue #916. */
-      serr = svn_repos__fs_change_rev_prop2
-        (db->resource->info->repos->repos,
-         db->resource->info->root.rev,
-         db->resource->info->repos->username,
-         propname, value,
-         db->authz_read_func,
-         db->authz_read_baton,
-         db->resource->pool);
+      serr = svn_repos_fs_change_rev_prop2(db->resource->info->repos->repos,
+                                           db->resource->info->root.rev,
+                                           db->resource->info->repos->username,
+                                           propname, value,
+                                           db->authz_read_func,
+                                           db->authz_read_baton,
+                                           db->resource->pool);
   else
     serr = svn_repos_fs_change_node_prop(db->resource->info->root.root,
                                          get_repos_path(db->resource->info),
@@ -385,14 +382,13 @@ static dav_error *dav_svn_db_remove(dav_db *db, const dav_prop_name *name)
          not a working resource!  But this is how we currently
          (hackily) allow the svn client to change unversioned rev
          props.  See issue #916. */
-      serr = svn_repos__fs_change_rev_prop2
-        (db->resource->info->repos->repos,
-         db->resource->info->root.rev,
-         db->resource->info->repos->username,
-         propname, NULL,
-         db->authz_read_func,
-         db->authz_read_baton,
-         db->resource->pool);
+      serr = svn_repos_fs_change_rev_prop2(db->resource->info->repos->repos,
+                                           db->resource->info->root.rev,
+                                           db->resource->info->repos->username,
+                                           propname, NULL,
+                                           db->authz_read_func,
+                                           db->authz_read_baton,
+                                           db->resource->pool);
   else
     serr = svn_repos_fs_change_node_prop(db->resource->info->root.root,
                                          get_repos_path(db->resource->info),
@@ -413,6 +409,7 @@ static int dav_svn_db_exists(dav_db *db, const dav_prop_name *name)
   const char *propname;
   svn_string_t *propval;
   svn_error_t *serr;
+  int retval;
 
   /* get the repos-local name */
   get_repos_propname(db, name, &propname);
@@ -427,12 +424,12 @@ static int dav_svn_db_exists(dav_db *db, const dav_prop_name *name)
       serr = svn_fs_txn_prop(&propval, db->resource->info->root.txn,
                              propname, db->p);
     else
-      serr = svn_repos__fs_revision_prop(&propval,
-                                         db->resource->info->repos->repos,
-                                         db->resource->info->root.rev,
-                                         propname,
-                                         db->authz_read_func,
-                                         db->authz_read_baton, db->p);
+      serr = svn_repos_fs_revision_prop(&propval,
+                                        db->resource->info->repos->repos,
+                                        db->resource->info->root.rev,
+                                        propname,
+                                        db->authz_read_func,
+                                        db->authz_read_baton, db->p);
   else
     serr = svn_fs_node_prop(&propval, db->resource->info->root.root,
                             get_repos_path(db->resource->info),
@@ -440,7 +437,9 @@ static int dav_svn_db_exists(dav_db *db, const dav_prop_name *name)
 
   /* ### try and dispose of the value? */
 
-  return serr == NULL && propval != NULL;
+  retval = (serr == NULL && propval != NULL);
+  svn_error_clear(serr);
+  return retval;
 }
 
 static void get_name(dav_db *db, dav_prop_name *pname)
@@ -485,7 +484,7 @@ static dav_error *dav_svn_db_first_name(dav_db *db, dav_prop_name *pname)
                                        db->resource->info->root.txn,
                                        db->p);
           else
-            serr = svn_repos__fs_revision_proplist
+            serr = svn_repos_fs_revision_proplist
               (&db->props,
                db->resource->info->repos->repos,
                db->resource->info->root.rev,
